from django.urls import reverse
from rest_framework import status
from rest_framework.exceptions import ValidationError
from ponos.models import Workflow
from arkindex_common.enums import DataImportMode
from arkindex.project.tests import FixtureTestCase
from arkindex.dataimport.models import Repository, DataImport


class TestRepositories(FixtureTestCase):

    def setUp(self):
        super().setUp()
        self.creds = self.user.credentials.get()
        self.repo = self.creds.repos.get()
        self.rev = self.repo.revisions.get()

    def test_delete_credentials_null(self):
        """
        Check deleting OAuthCredentials do not delete repositories and revisions
        """
        self.creds.delete()
        self.assertTrue(Repository.objects.filter(url='http://gitlab/repo').exists())
        self.repo.refresh_from_db()
        self.assertTrue(self.repo.revisions.exists())

    def test_no_credentials_no_import(self):
        """
        Check Repository imports do not start without credentials
        """
        self.repo.credentials = None
        self.repo.save()
        self.assertEqual(Workflow.objects.count(), 0)

        di = DataImport.objects.create(
            mode=DataImportMode.Repository,
            revision=self.rev,
            creator=self.superuser,
            corpus=self.corpus,
        )

        with self.assertRaises(ValidationError):
            di.start()

        with self.assertRaises(ValidationError):
            di.retry()

        self.assertEqual(Workflow.objects.count(), 0)

    def test_start_no_credentials(self):
        """
        Test the repository start endpoint fails without credentials
        """
        self.client.force_login(self.superuser)
        self.repo.credentials = None
        self.repo.save()
        self.assertFalse(self.repo.enabled)
        self.assertEqual(Workflow.objects.count(), 0)

        resp = self.client.get(reverse('api:repository-import', kwargs={'pk': str(self.repo.id)}))
        self.assertEqual(resp.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(Workflow.objects.count(), 0)
